@@ -1,4 +1,4 @@
-/* $Id: cgps.c 4547 2007-12-13 01:38:22Z esr $ */
+/* $Id: cgps.c 5053 2009-01-21 11:44:35Z esr $ */
 /*
  * Copyright (c) 2005 Jeff Francis <jeff@gritch.org>
  *
@@ -84,12 +84,14 @@
 
 #include <sys/types.h>
 #include <sys/select.h>
+#ifndef S_SPLINT_S
 #include <sys/socket.h>
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <math.h>
 #include <errno.h>
 #include <assert.h>
@@ -117,6 +119,7 @@ static WINDOW *datawin, *satellites, *messages;
 
 static int raw_flag=0;
 static int silent_flag=0;
+static int magnetic_flag=0;
 static int fixclear_flag=0;
 static int compass_flag=0;
 static int got_gps_type=0;
@@ -124,6 +127,55 @@ static char gps_type[26];
 static int window_length;
 static int display_sats;
 
+/* Convert true heading to magnetic.  Taken from the Aviation
+   Formulary v1.43.  Valid to within two degrees within the
+   continiental USA except for the following airports: MO49 MO86 MO50
+   3K6 02K and KOOA.  AK correct to better than one degree.  Western
+   Europe correct to within 0.2 deg.
+
+   If you're not in one of these areas, I apologize, I don't have the
+   math to compute your varation.  This is obviously extremely
+   floating-point heavy, so embedded people, beware of using.
+
+   Note that there are issues with using magnetic heading.  This code
+   does not account for the possibility of travelling into or out of
+   an area of valid calculation beyond forcing the magnetic conversion
+   off.  A better way to communicate this to the user is probably
+   desirable (in case the don't notice the subtle change from "(mag)"
+   to "(true)" on their display).
+ */
+static float true2magnetic(double lat, double lon, double heading)
+{
+  /* Western Europe */
+  /*@ -evalorder +relaxtypes @*/
+  if((lat > 36.0) && (lat < 68.0) &&
+     (lon > -10.0) && (lon < 28.0)) {
+    return( 10.4768771667158 - (0.507385322418858 * lon) + (0.00753170031703826 * pow(lon, 2))
+	    - (1.40596203924748e-05 * pow(lon, 3)) - (0.535560699962353 * lat)
+	    + (0.0154348808069955 * lat * lon) - (8.07756425110592e-05 * lat * pow(lon, 2))
+	    + (0.00976887198864442 * pow(lat, 2)) - (0.000259163929798334 * lon * pow(lat, 2))
+	    - (3.69056939266123e-05 * pow(lat, 3)) + heading);
+  }
+  lon=0.0-lon;
+  /* USA */
+  if((lat > 24.0) && (lat < 50.0) &&
+     (lon > 66.0) && (lon < 125.0)) {
+    return( (-65.6811) + (0.99 * lat) + (0.0128899 * pow(lat, 2)) - (0.0000905928 * pow(lat, 3)) + (2.87622 * lon)
+	    - (0.0116268 * lat * lon) - (0.00000603925 * lon * pow(lat, 2)) - (0.0389806 * pow(lon, 2))
+	    - (0.0000403488 * lat * pow(lon, 2)) + (0.000168556 * pow(lon, 3)) + heading);
+  }
+  /* AK */
+  if((lat > 54.0) &&
+     (lon > 130.0) && (lon < 172.0)) {
+    return( 618.854 + (2.76049 * lat) - (0.556206 * pow(lat, 2)) + (0.00251582 * pow(lat, 3)) - (12.7974 * lon)
+	    + (0.408161 * lat * lon) + (0.000434097 * lon * pow(lat, 2)) - (0.00602173 * pow(lon, 2))
+	    - (0.00144712 * lat * pow(lon, 2)) + (0.000222521 * pow(lon, 3)) + heading);
+  }
+  magnetic_flag=0;
+  return(heading);
+  /*@ +evalorder -relaxtypes @*/
+}
+
 /* Function to call when we're all done.  Does a bit of clean-up. */
 static void die(int sig UNUSED)
 {
@@ -347,7 +399,7 @@ static void update_compass_panel(struct gps_data_t *gpsdata,
 
   /* Fill in the heading. */
   if (isnan(gpsdata->fix.track)==0) {
-    (void)snprintf(scr, sizeof(scr), "%.1f", gpsdata->fix.track);
+    (void)snprintf(scr, sizeof(scr), "%.1f degrees", gpsdata->fix.track);
   } else
     (void)snprintf(scr, sizeof(scr), "n/a");
   (void)mvwprintw(datawin, 2, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
@@ -487,7 +539,11 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
 
   /* Fill in the heading. */
   if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0)
-    (void)snprintf(scr, sizeof(scr), "%.1f degrees", gpsdata->fix.track);
+    if(magnetic_flag==0) {
+      (void)snprintf(scr, sizeof(scr), "%.1f deg (true)", gpsdata->fix.track);
+    } else {
+      (void)snprintf(scr, sizeof(scr), "%.1f deg (mag) ", true2magnetic(gpsdata->fix.latitude, gpsdata->fix.longitude, gpsdata->fix.track));
+    }
   else
     (void)snprintf(scr, sizeof(scr), "n/a");
   (void)mvwprintw(datawin, 6, DATAWIN_VALUE_OFFSET, "%-*s", 27, scr);
@@ -555,7 +611,7 @@ static void update_gps_panel(struct gps_data_t *gpsdata,
 
       /* Fill in the estimated track error. */
       if (isnan(gpsdata->fix.epd)==0)
-	(void)snprintf(scr, sizeof(scr), "+/- %.1f deg", (gpsdata->fix.epd));
+	(void)snprintf(scr, sizeof(scr), "+/- %d deg", (int)(gpsdata->fix.epd));
       else
         (void)snprintf(scr, sizeof(scr), "n/a");
       (void)mvwprintw(datawin, 12, DATAWIN_VALUE_OFFSET + 5, "%-*s", 22, scr);
@@ -597,6 +653,8 @@ static void usage( char *prog)
 		"		d = DD.dddddd\n"
 		"		m = DD MM.mmmm'\n"
 		"		s = DD MM' SS.sss\"\n"
+		" -m      Display heading as the estimated magnetic heading\n"
+		"         Valid only for USA (Lower 48 + AK) and Western Europe.\n"
 		, prog);
 
   exit(1);
@@ -614,8 +672,11 @@ int main(int argc, char *argv[])
   int data;
 
   /* Process the options.  Print help if requested. */
-  while ((option = getopt(argc, argv, "hVl:sj")) != -1) {
+  while ((option = getopt(argc, argv, "hVl:sjm")) != -1) {
     switch (option) {
+    case 'm':
+      magnetic_flag=1;
+      break;
     case 's':
       silent_flag=1;
       break;
@@ -623,7 +684,7 @@ int main(int argc, char *argv[])
       fixclear_flag=1;
       break;
     case 'V':
-      (void)fprintf(stderr, "SVN ID: $Id: cgps.c 4547 2007-12-13 01:38:22Z esr $ \n");
+      (void)fprintf(stderr, "SVN ID: $Id: cgps.c 5053 2009-01-21 11:44:35Z esr $ \n");
       exit(0);
     case 'l':
       switch ( optarg[0] ) {
