@@ -1,4 +1,4 @@
-/* $Id: drivers.c 4663 2008-01-20 23:35:32Z garyemiller $ */
+/* $Id: drivers.c 5130 2009-02-08 18:09:31Z esr $ */
 #include <sys/types.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
@@ -12,6 +12,7 @@
 
 #include "gpsd_config.h"
 #include "gpsd.h"
+#include "bits.h"	/* for getbeuw(), to extract big-endiamn words */
 
 extern struct gps_type_t zodiac_binary;
 extern struct gps_type_t ubx_binary;
@@ -73,6 +74,14 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 #else
 	return 0;
 #endif /* GARMIN_ENABLE */
+    } else if (session->packet.type == UBX_PACKET) {
+	gpsd_report(LOG_WARN, "UBX packet seen when NMEA expected.\n");
+#ifdef UBX_ENABLE
+	(void)gpsd_switch_driver(session, "uBlox UBX binary");
+	return ubx_parse(session, session->packet.outbuffer, session->packet.outbuflen);
+#else
+	return 0;
+#endif /* UBX_ENABLE */
     } else if (session->packet.type == NMEA_PACKET) {
 	gps_mask_t st = 0;
 #ifdef GARMINTXT_ENABLE
@@ -84,64 +93,40 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 		}
 	}
 #endif /* GARMINTXT_ENABLE */
+
+#ifdef OCEANSERVER_ENABLE
+	if (strncmp((char *)session->packet.outbuffer, "$C", 2)==0 || strncmp((char *)session->packet.outbuffer, "$OHPR", 5)==0) {
+		(void)gpsd_switch_driver(session, "OceanServer Digital Compas OS5000");
+		return  1;
+	}
+#endif /* OCEANSERVER_ENABLE */
+
 	gpsd_report(LOG_IO, "<= GPS: %s", session->packet.outbuffer);
+
 	if ((st=nmea_parse((char *)session->packet.outbuffer, session))==0) {
 #ifdef NON_NMEA_ENABLE
 	    struct gps_type_t **dp;
 
 	    /* maybe this is a trigger string for a driver we know about? */
+#ifdef UBX_ENABLE
+	    if(strncmp((char *)session->packet.outbuffer, "$GPTXT,01,01,02,MOD", 19)==0) {
+		ubx_catch_model(session, session->packet.outbuffer, session->packet.outbuflen);
+		(void)gpsd_switch_driver(session, "uBlox UBX binary");
+		return 0;
+	    }
+#endif /* UBX_ENABLE */
 	    for (dp = gpsd_drivers; *dp; dp++) {
 		char	*trigger = (*dp)->trigger;
 
 		if (trigger!=NULL && strncmp((char *)session->packet.outbuffer, trigger, strlen(trigger))==0 && isatty(session->gpsdata.gps_fd)!=0) {
 		    gpsd_report(LOG_PROG, "found %s.\n", trigger);
 		    (void)gpsd_switch_driver(session, (*dp)->type_name);
-		    return 1;
+		    return DEVICEID_SET;
 		}
 	    }
 #endif /* NON_NMEA_ENABLE */
 	    gpsd_report(LOG_WARN, "unknown sentence: \"%s\"\n", session->packet.outbuffer);
 	}
-#ifdef NMEADISC
-	if (session->ldisc == 0) {
-	    uid_t old;
-	    int ldisc = NMEADISC;
-
-#ifdef TIOCSTSTAMP
-	    struct tstamps tstamps;
-#ifdef PPS_ON_CTS
-	    tstamps.ts_set |= TIOCM_CTS;
-#else /*!PPS_ON_CTS */
-	    tstamps.ts_set |= TIOCM_CAR;
-#endif /* PPS_ON_CTS */
-	    tstamps.ts_clr = 0;
-
-	    old = geteuid();
-	    if (seteuid(0) == -1)
-		gpsd_report(LOG_WARN, "can't seteuid(0) - %s", strerror(errno));
-	    else
-		gpsd_report(LOG_WARN, "seteuid(0) to enable timestamping\n");
-	    if (ioctl(session->gpsdata.gps_fd, TIOCSTSTAMP, &tstamps) < 0)
-		gpsd_report(LOG_WARN, "can't set kernel timestamping: %s\n",
-		    strerror(errno));
-	    else
-		gpsd_report(LOG_WARN, "activated kernel timestamping\n");
-#endif /* TIOCSTSTAMP */
-	    if (ioctl(session->gpsdata.gps_fd, TIOCSETD, &ldisc) == -1)
-		gpsd_report(LOG_WARN, "can't set nmea discipline: %s\n",
-		    strerror(errno));
-	    else
-		gpsd_report(LOG_WARN, "activated nmea discipline\n");
-	/* this is a flag that shows if we've tried the setup */
-	session->ldisc = NMEADISC;
-
-	if (old){
-	    gpsd_report(LOG_WARN, "giving up euid 0\n");
-	    (void)seteuid(old);
-	}
-	gpsd_report(LOG_WARN, "running with effective user ID %d\n", geteuid());
-	}
-#endif /*NMEADISC */
 #ifdef NTPSHM_ENABLE
 	/* this magic number is derived from observation */
 	if (session->context->enable_ntpshm &&
@@ -158,6 +143,9 @@ gps_mask_t nmea_parse_input(struct gps_device_t *session)
 
 static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 {
+    /* change this guard if the probe count goes up */ 
+    if (seq <= 8)
+	gpsd_report(LOG_WARN, "=> Probing device subtype %d\n", seq);
     /*
      * The reason for splitting these probes up by packet sequence
      * number, interleaving them with the first few packet receives,
@@ -178,29 +166,34 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
     switch (seq) {
 #ifdef SIRF_ENABLE
     case 0:
+	/* probe for Garmin serial GPS -- expect $PGRMC followed by data*/
+	(void)nmea_send(session, "$PGRMCE");
+	break;
+#endif /* SIRF_ENABLE */
+#ifdef NMEA_ENABLE
+    case 1:
 	/*
 	 * We used to try to probe for SiRF by issuing "$PSRF105,1"
 	 * and expecting "$Ack Input105.".  But it turns out this
 	 * only works for SiRF-IIs; SiRF-I and SiRF-III don't respond.
 	 * Thus the only reliable probe is to try to flip the SiRF into
 	 * binary mode, cluing in the library to revert it on close.
+	 *
+	 * SiRFs dominate the GPS-mouse market, so we used to put this test 
+	 * first. Unfortunately this causes problems for gpsctl, as it cannot
+	 * select the NMEA driver without switchining the device back to
+	 * binary mode!  Fix this if we ever find a nondisruptive probe string.
 	 */
-	(void)nmea_send(session->gpsdata.gps_fd,
+	(void)nmea_send(session,
 			"$PSRF100,0,%d,%d,%d,0",
 			session->gpsdata.baudrate,
 			9-session->gpsdata.stopbits,
 			session->gpsdata.stopbits);
 	session->back_to_nmea = true;
 	break;
-#endif /* SIRF_ENABLE */
-#ifdef NMEA_ENABLE
-    case 1:
-	/* probe for Garmin serial GPS -- expect $PGRMC followed by data*/
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMCE");
-	break;
     case 2:
 	/* probe for the FV-18 -- expect $PFEC,GPint followed by data */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PFEC,GPint");
+	(void)nmea_send(session, "$PFEC,GPint");
 	break;
 #endif /* NMEA_ENABLE */
 #ifdef EVERMORE_ENABLE
@@ -214,21 +207,33 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 #ifdef ITRAX_ENABLE
     case 4:
 	/* probe for iTrax, looking for "$PFST,OK" */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PFST");
+	(void)nmea_send(session, "$PFST");
 	break;
 #endif /* ITRAX_ENABLE */
 #ifdef GPSCLOCK_ENABLE
     case 5:
-	/* probe for Furuno Electric GH-79L4-N (GPSClock) */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PFEC,GPsrq");
+	/* probe for Furuno Electric GH-79L4-N (GPSClock); expect $PFEC,GPssd */
+	(void)nmea_send(session, "$PFEC,GPsrq");
 	break;
 #endif /* GPSCLOCK_ENABLE */
 #ifdef ASHTECH_ENABLE
     case 6:
-	/* probe for Ashtech -- expect $PASHR */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHQ,RID");
+	/* probe for Ashtech -- expect $PASHR,RID */
+	(void)nmea_send(session, "$PASHQ,RID");
 	break;
 #endif /* ASHTECH_ENABLE */
+#ifdef UBX_ENABLE
+    case 7:
+	/* probe for UBX -- query software version */
+	(void)ubx_write(session->gpsdata.gps_fd, 0x0au, 0x04, NULL, 0);
+	break;
+#endif /* UBX_ENABLE */
+#ifdef MKT3301_ENABLE
+    case 8:
+	/* probe for MKT-3301 -- expect $PMTK705 */
+	(void)nmea_send(session, "$PMTK605");
+	break;
+#endif /* MKT3301_ENABLE */
     default:
 	break;
     }
@@ -236,8 +241,10 @@ static void nmea_probe_subtype(struct gps_device_t *session, unsigned int seq)
 
 static struct gps_type_t nmea = {
     .type_name      = "Generic NMEA",	/* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
     .trigger	    = NULL,		/* it's the default */
     .channels       = 12,		/* consumer-grade GPS */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = nmea_probe_subtype, /* probe for special types */
@@ -265,6 +272,17 @@ static struct gps_type_t nmea = {
  *
  **************************************************************************/
 
+static void garmin_mode_switch(struct gps_device_t *session, int mode)
+/* only does anything iin one direction, going to Garmin binary driver */
+{
+    if (mode == MODE_BINARY) {
+	(void)nmea_send(session, "$PGRMC1,1,2,1,,,,2,W,N");
+	(void)nmea_send(session, "$PGRMI,,,,,,,R");
+	(void)usleep(333);	/* standard Garmin settling time */
+	session->gpsdata.driver_mode = MODE_BINARY;
+    }
+}
+
 #ifdef ALLOW_RECONFIGURE
 static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int seq)
 {
@@ -279,33 +297,33 @@ static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int
          * with a 160ms low state.
          * NOTE: new PPS only takes effect after next power cycle
          */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMC,A,,100,,,,,,A,,1,2,1,30");
+	(void)nmea_send(session, "$PGRMC,A,,100,,,,,,A,,1,2,1,30");
 	break;
     case 1:
 	/* once a sec, no averaging, NMEA 2.3, WAAS */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMC1,1,1,1,,,,2,W,N");
+	(void)nmea_send(session, "$PGRMC1,1,1,1,,,,2,W,N");
 	break;
     case 2:
 	/* get some more config info */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMC1E");
+	(void)nmea_send(session, "$PGRMC1E");
 	break;
     case 3:
 	/* turn off all output except GGA */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,,2");
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,GPGGA,1");
+	(void)nmea_send(session, "$PGRMO,,2");
+	(void)nmea_send(session, "$PGRMO,GPGGA,1");
 	break;
     case 4:
 	/* enable GPGGA, GPGSA, GPGSV, GPRMC on Garmin serial GPS */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,GPGSA,1");
+	(void)nmea_send(session, "$PGRMO,GPGSA,1");
 	break;
     case 5:
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,GPGSV,1");
+	(void)nmea_send(session, "$PGRMO,GPGSV,1");
 	break;
     case 6:
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,GPRMC,1");
+	(void)nmea_send(session, "$PGRMO,GPRMC,1");
 	break;
     case 7:
-	(void)nmea_send(session->gpsdata.gps_fd, "$PGRMO,PGRME,1");
+	(void)nmea_send(session, "$PGRMO,PGRME,1");
 	break;
     }
 }
@@ -313,8 +331,10 @@ static void garmin_nmea_configurator(struct gps_device_t *session, unsigned int
 
 static struct gps_type_t garmin = {
     .type_name      = "Garmin Serial",	/* full name of type */
+    .packet_type    = GARMIN_PACKET,	/* associated lexer packet type */
     .trigger	    = "$PGRMC,",	/* Garmin private */
     .channels       = 12,		/* not used by this driver */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = NULL,		/* no further querying */
@@ -325,7 +345,7 @@ static struct gps_type_t garmin = {
     .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
     .rtcm_writer    = NULL,		/* some do, some don't, skip for now */
     .speed_switcher = NULL,		/* no speed switcher */
-    .mode_switcher  = NULL,		/* no mode switcher */
+    .mode_switcher  = garmin_mode_switch,	/* mode switcher */
     .rate_switcher  = NULL,		/* no sample-rate switcher */
     .cycle_chars    = -1,		/* not relevant, no rate switch */
 #ifdef ALLOW_RECONFIGURE
@@ -348,35 +368,37 @@ static void ashtech_configure(struct gps_device_t *session, unsigned int seq)
 {
     if (seq == 0){
 	/* turn WAAS on. can't hurt... */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,WAS,ON");
+	(void)nmea_send(session, "$PASHS,WAS,ON");
 	/* reset to known output state */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,ALL,A,OFF");
+	(void)nmea_send(session, "$PASHS,NME,ALL,A,OFF");
 	/* then turn on some useful sentences */
 #ifdef ASHTECH_NOTYET
 	/* we could parse these, but they're oversize so they get dropped */
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,POS,A,ON");
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,SAT,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,POS,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,SAT,A,ON");
 #else
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GGA,A,ON");
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GSA,A,ON");
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,GSV,A,ON");
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,RMC,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,GGA,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,GSA,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,GSV,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,RMC,A,ON");
 #endif
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHS,NME,ZDA,A,ON");
+	(void)nmea_send(session, "$PASHS,NME,ZDA,A,ON");
     }
 }
 #endif /* ALLOW_RECONFIGURE */
 
 static void ashtech_ping(struct gps_device_t *session)
 {
-	(void)nmea_send(session->gpsdata.gps_fd, "$PASHQ,RID");
+	(void)nmea_send(session, "$PASHQ,RID");
 }
 
 static struct gps_type_t ashtech = {
     .type_name      = "Ashtech",	/* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
     .trigger	    = "$PASHR,RID,",	/* Ashtech receivers respond thus */
     .channels       = 24,		/* not used, GG24 has 24 channels */
-    .probe_wakeup   = ashtech_ping,	/* no wakeup to be done before hunt */
+    .control_send   = nmea_write,	/* how to send control strings */
+    .probe_wakeup   = ashtech_ping,	/* wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = NULL,		/* to be sent unconditionally */
 #ifdef ALLOW_RECONFIGURE
@@ -412,15 +434,17 @@ static void fv18_configure(struct gps_device_t *session, unsigned int seq)
      * Suppress GLL and VTG.  Enable ZDA so dates will be accurate for replay.
      */
     if (seq == 0)
-	(void)nmea_send(session->gpsdata.gps_fd,
+	(void)nmea_send(session,
 		    "$PFEC,GPint,GSA01,DTM00,ZDA01,RMC01,GLL00,VTG00,GSV05");
 }
 #endif /* ALLOW_RECONFIGURE */
 
 static struct gps_type_t fv18 = {
     .type_name      = "San Jose Navigation FV18",	/* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
     .trigger	    = "$PFEC,GPint,",	/* FV18s should echo the probe */
     .channels       = 12,		/* not used by this driver */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = NULL,		/* to be sent unconditionally */
@@ -467,8 +491,10 @@ static void gpsclock_probe_subtype(struct gps_device_t *session, unsigned int se
 
 static struct gps_type_t gpsclock = {
     .type_name      = "Furuno Electric GH-79L4",	/* full name of type */
-    .trigger	    = "$PFEC,GPssd",	/* GPSclock should echo the probe */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
+    .trigger	    = "$PFEC,GPssd",	/* GPSclock should return this */
     .channels       = 12,		/* not used by this driver */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
     .probe_detect   = NULL,		/* no probe */
     .probe_subtype  = gpsclock_probe_subtype, /* to be sent unconditionally */
@@ -509,7 +535,7 @@ static void tripmate_probe_subtype(struct gps_device_t *session, unsigned int se
 {
     /* TripMate requires this response to the ASTRAL it sends at boot time */
     if (seq == 0)
-	(void)nmea_send(session->gpsdata.gps_fd, "$IIGPQ,ASTRAL");
+	(void)nmea_send(session, "$IIGPQ,ASTRAL");
 }
 
 #ifdef ALLOW_RECONFIGURE
@@ -517,14 +543,16 @@ static void tripmate_configurator(struct gps_device_t *session, unsigned int seq
 {
     /* stop it sending PRWIZCH */
     if (seq == 0)
-	(void)nmea_send(session->gpsdata.gps_fd, "$PRWIILOG,ZCH,V,,");
+	(void)nmea_send(session, "$PRWIILOG,ZCH,V,,");
 }
 #endif /* ALLOW_RECONFIGURE */
 
 static struct gps_type_t tripmate = {
     .type_name     = "Delorme TripMate",	/* full name of type */
+    .packet_type   = NMEA_PACKET,		/* lexer packet type */
     .trigger       ="ASTRAL",			/* tells us to switch */
     .channels      = 12,			/* consumer-grade GPS */
+    .control_send  = nmea_write,		/* how to send control strings */
     .probe_wakeup  = NULL,			/* no wakeup before hunt */
     .probe_detect  = NULL,			/* no probe */
     .probe_subtype = tripmate_probe_subtype,	/* send unconditionally */
@@ -581,8 +609,10 @@ static void earthmate_probe_subtype(struct gps_device_t *session, unsigned int s
 /*@ -redef @*/
 static struct gps_type_t earthmate = {
     .type_name     = "Delorme EarthMate (pre-2003, Zodiac chipset)",
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
     .trigger       = "EARTHA",			/* Earthmate trigger string */
     .channels      = 12,			/* not used by NMEA parser */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup  = NULL,			/* no wakeup to be done before hunt */
     .probe_detect  = NULL,			/* no probe */
     .probe_subtype = earthmate_probe_subtype,	/* switch us to Zodiac mode */
@@ -752,10 +782,12 @@ static bool tnt_probe(struct gps_device_t *session)
   return false;
 }
 
-struct gps_type_t trueNorth = {
+static struct gps_type_t trueNorth = {
     .type_name      = "True North",	/* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
     .trigger	    = " TNT1500",
     .channels       = 0,		/* not an actual GPS at all */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup   = NULL,		/* this will become a real method */
     .probe_detect   = tnt_probe,	/* probe by sending ID query */
     .probe_subtype  = tnt_probe_subtype,/* probe for True North Digital Compass */
@@ -776,27 +808,104 @@ struct gps_type_t trueNorth = {
     .cycle	    = 20,		/* updates per second */
 };
 #endif
-#ifdef RTCM104_ENABLE
+
+#ifdef OCEANSERVER_ENABLE
 /**************************************************************************
+ * OceanServer - Digital Compass, OS5000 Series
  *
- * RTCM-104, used for broadcasting DGPS corrections and by DGPS radios
+ * More info: http://www.ocean-server.com/download/OS5000_Compass_Manual.pdf
+ *
+ * This is a digital compass which uses magnetometers to measure the
+ * strength of the earth's magnetic field. Based on these measurements
+ * it provides a compass heading using NMEA formatted output strings.
+ * This is useful to supplement the heading provided by another GPS
+ * unit. A GPS heading is unreliable at slow speed or no speed.
  *
  **************************************************************************/
 
-static gps_mask_t rtcm104_analyze(struct gps_device_t *session)
+static int oceanserver_send(int fd, const char *fmt, ... )
 {
-    rtcm_unpack(&session->gpsdata.rtcm, (char *)session->packet.isgps.buf);
-    gpsd_report(LOG_RAW, "RTCM packet type 0x%02x length %d words: %s\n",
-		session->gpsdata.rtcm.type,
-		session->gpsdata.rtcm.length+2,
-		gpsd_hexdump(session->packet.isgps.buf, (session->gpsdata.rtcm.length+2)*sizeof(isgps30bits_t)));
-    return RTCM_SET;
+    int status;
+    char buf[BUFSIZ];
+    va_list ap;
+
+    va_start(ap, fmt) ;
+    (void)vsnprintf(buf, sizeof(buf)-5, fmt, ap);
+    va_end(ap);
+    strlcat(buf, "", BUFSIZ);
+    status = (int)write(fd, buf, strlen(buf));
+    tcdrain(fd);
+    if (status == (int)strlen(buf)) {
+	gpsd_report(LOG_IO, "=> GPS: %s\n", buf);
+	return status;
+    } else {
+	gpsd_report(LOG_WARN, "=> GPS: %s FAILED\n", buf);
+	return -1;
+    }
 }
 
-static struct gps_type_t rtcm104 = {
-    .type_name     = "RTCM104",		/* full name of type */
+#ifdef ALLOW_RECONFIGURE
+static void oceanserver_configure(struct gps_device_t *session, unsigned int seq)
+{
+    if (seq == 0){
+	/* report in NMEA format */
+	(void)oceanserver_send(session->gpsdata.gps_fd, "2\n");
+	/* ship all fields */
+	(void)oceanserver_send(session->gpsdata.gps_fd, "X2047");
+    }
+}
+#endif /* ALLOW_RECONFIGURE */
+
+static struct gps_type_t oceanServer = {
+    .type_name      = "OceanServer Digital Compass OS5000", /* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
+    .trigger	    = "$C,",
+    .channels       = 0,		/* not an actual GPS at all */
+    .control_send   = nmea_write,	/* how to send control strings */
+    .probe_wakeup   = NULL,
+    .probe_detect   = NULL,
+    .probe_subtype  = NULL,
+#ifdef ALLOW_RECONFIGURE
+    .configurator   = oceanserver_configure,
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet     = generic_get,	/* how to get a packet */
+    .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
+    .rtcm_writer    = NULL,		/* Don't send */
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no wrapup */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+#ifdef ALLOW_RECONFIGURE
+    .revert	    = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 20,		/* updates per second */
+};
+#endif
+
+#ifdef RTCM104V2_ENABLE
+/**************************************************************************
+ *
+ * RTCM-104 (v2), used for broadcasting DGPS corrections and by DGPS radios
+ *
+ **************************************************************************/
+
+static gps_mask_t rtcm104v2_analyze(struct gps_device_t *session)
+{
+    rtcm2_unpack(&session->gpsdata.rtcm2, (char *)session->packet.isgps.buf);
+    gpsd_report(LOG_RAW, "RTCM 2.x packet type 0x%02x length %d words: %s\n",
+	session->gpsdata.rtcm2.type, session->gpsdata.rtcm2.length+2,
+	gpsd_hexdump_wrapper(session->packet.isgps.buf,
+	    (session->gpsdata.rtcm2.length+2)*sizeof(isgps30bits_t), LOG_RAW));
+    return RTCM2_SET;
+}
+
+static struct gps_type_t rtcm104v2 = {
+    .type_name     = "RTCM104V2",	/* full name of type */
+    .packet_type   = RTCM2_PACKET,	/* associated lexer packet type */
     .trigger       = NULL,		/* no recognition string */
     .channels      = 0,			/* not used */
+    .control_send  = NULL,		/* how to send control strings */
     .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
     .probe_detect  = NULL,		/* no probe */
     .probe_subtype = NULL,		/* no subtypes */
@@ -804,7 +913,7 @@ static struct gps_type_t rtcm104 = {
     .configurator  = NULL,		/* no configurator */
 #endif /* ALLOW_RECONFIGURE */
     .get_packet    = generic_get,	/* how to get a packet */
-    .parse_packet  = rtcm104_analyze,	/*  */
+    .parse_packet  = rtcm104v2_analyze,	/*  */
     .rtcm_writer   = NULL,		/* don't send RTCM data,  */
     .speed_switcher= NULL,		/* no speed switcher */
     .mode_switcher = NULL,		/* no mode switcher */
@@ -816,7 +925,51 @@ static struct gps_type_t rtcm104 = {
     .wrapup	   = NULL,		/* no wrapup code */
     .cycle	   = 1,			/* updates every second */
 };
-#endif /* RTCM104_ENABLE */
+#endif /* RTCM104V2_ENABLE */
+#ifdef RTCM104V3_ENABLE
+/**************************************************************************
+ *
+ * RTCM-104 (v3), used for broadcasting DGPS corrections and by DGPS radios
+ *
+ **************************************************************************/
+
+static gps_mask_t rtcm104v3_analyze(struct gps_device_t *session)
+{
+    uint length = getbeuw(session->packet.inbuffer, 1);
+    uint type = getbeuw(session->packet.inbuffer, 3) >> 4;
+
+    gpsd_report(LOG_RAW, "RTCM 3.x packet type %d length %d words: %s\n",
+	type, length, gpsd_hexdump_wrapper(session->packet.inbuffer,
+	    (size_t)(session->gpsdata.rtcm3.length), LOG_RAW));
+    return RTCM3_SET;
+}
+
+static struct gps_type_t rtcm104v3 = {
+    .type_name     = "RTCM104V3",	/* full name of type */
+    .packet_type   = RTCM3_PACKET,	/* associated lexer packet type */
+    .trigger       = NULL,		/* no recognition string */
+    .channels      = 0,			/* not used */
+    .control_send  = NULL,		/* how to send control strings */
+    .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
+    .probe_detect  = NULL,		/* no probe */
+    .probe_subtype = NULL,		/* no subtypes */
+#ifdef ALLOW_RECONFIGURE
+    .configurator  = NULL,		/* no configurator */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet    = generic_get,	/* how to get a packet */
+    .parse_packet  = rtcm104v3_analyze,	/*  */
+    .rtcm_writer   = NULL,		/* don't send RTCM data,  */
+    .speed_switcher= NULL,		/* no speed switcher */
+    .mode_switcher = NULL,		/* no mode switcher */
+    .rate_switcher = NULL,		/* no sample-rate switcher */
+    .cycle_chars   = -1,		/* not relevant, no rate switch */
+#ifdef ALLOW_RECONFIGURE
+    .revert	   = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	   = NULL,		/* no wrapup code */
+    .cycle	   = 1,			/* updates every second */
+};
+#endif /* RTCM104V3_ENABLE */
 
 #ifdef GARMINTXT_ENABLE
 /**************************************************************************
@@ -834,8 +987,10 @@ static gps_mask_t garmintxt_parse_input(struct gps_device_t *session)
 
 static struct gps_type_t garmintxt = {
     .type_name     = "Garmin Simple Text",		/* full name of type */
+    .packet_type   = RTCM2_PACKET;	/* associated lexer packet type */
     .trigger       = NULL,		/* no recognition string */
     .channels      = 0,			/* not used */
+    .control_send   = nmea_write,	/* how to send control strings */
     .probe_wakeup  = NULL,		/* no wakeup to be done before hunt */
     .probe_detect  = NULL,		/* no probe */
     .probe_subtype = NULL,		/* no subtypes */
@@ -857,10 +1012,105 @@ static struct gps_type_t garmintxt = {
 };
 #endif /* GARMINTXT_ENABLE */
 
+#ifdef MKT3301_ENABLE
+/**************************************************************************
+ *
+ * MKT-3301
+ *
+ **************************************************************************/
+const char* mkt_reasons[4] = {"Invalid", "Unsupported", "Valid but Failed", "Valid success"};
+
+gps_mask_t processMKT3301(int c UNUSED, char *field[], struct gps_device_t *session)
+{
+    int msg, reason;
+    gps_mask_t mask;
+    mask = 1; //ONLINE_SET;
+
+    switch(msg = atoi(&(field[0])[4]))
+    {
+	case 705: /*  */
+	    (void)strlcat(session->subtype,field[1],64);
+	    (void)strlcat(session->subtype,"-",64);
+	    (void)strlcat(session->subtype,field[2],64);
+	    return 0; /* return a unknown sentence, which will cause the driver switch */
+	case 001: /* ACK / NACK */
+	    reason = atoi(field[2]);
+	    if(atoi(field[1]) == -1)
+		gpsd_report(LOG_WARN, "MKT NACK: unknown sentence\n");
+	    else if(reason < 3)
+		gpsd_report(LOG_WARN, "MKT NACK: %s, reason: %s\n", field[1], mkt_reasons[reason]);
+	    else
+		gpsd_report(LOG_WARN, "MKT ACK: %s\n", field[1]);
+	    break;
+	default:
+	    return 0; /* ignore */
+    }
+    return mask;
+}
+
+#ifdef ALLOW_RECONFIGURE
+static void mkt3301_configure(struct gps_device_t *session, unsigned int seq)
+{
+/*
+0  NMEA_SEN_GLL,  GPGLL   interval - Geographic Position - Latitude longitude
+1  NMEA_SEN_RMC,  GPRMC   interval - Recommended Minimum Specific GNSS Sentence
+2  NMEA_SEN_VTG,  GPVTG   interval - Course Over Ground and Ground Speed
+3  NMEA_SEN_GGA,  GPGGA   interval - GPS Fix Data
+4  NMEA_SEN_GSA,  GPGSA   interval - GNSS DOPS and Active Satellites
+5  NMEA_SEN_GSV,  GPGSV   interval - GNSS Satellites in View
+6  NMEA_SEN_GRS,  GPGRS   interval - GNSS Range Residuals
+7  NMEA_SEN_GST,  GPGST   interval - GNSS Pseudorange Errors Statistics
+13 NMEA_SEN_MALM, PMTKALM interval - GPS almanac information
+14 NMEA_SEN_MEPH, PMTKEPH interval - GPS ephmeris information
+15 NMEA_SEN_MDGP, PMTKDGP interval - GPS differential correction information
+16 NMEA_SEN_MDBG, PMTKDBG interval – MTK debug information
+17 NMEA_SEN_ZDA,  GPZDA   interval - Time & Date
+18 NMEA_SEN_MCHN, PMTKCHN interval – GPS channel status
+
+"$PMTK314,1,1,1,1,1,5,1,1,0,0,0,0,0,0,0,0,0,1,0"
+
+*/
+    if(seq == 0) {
+	(void)nmea_send(session,"$PMTK320,0"); /* power save off */
+	(void)nmea_send(session,"$PMTK300,1000,0,0,0.0,0.0"); /* Fix interval */
+	(void)nmea_send(session,"$PMTK314,0,1,0,1,1,5,1,1,0,0,0,0,0,0,0,0,0,1,0");
+	(void)nmea_send(session,"$PMTK301,2"); /* DGPS is WAAS */
+	(void)nmea_send(session,"$PMTK313,1"); /* SBAS enable */
+    }
+}
+#endif /* ALLOW_RECONFIGURE */
+
+static struct gps_type_t mkt3301 = {
+    .type_name      = "MKT-3301",	/* full name of type */
+    .packet_type    = NMEA_PACKET,	/* associated lexer packet type */
+    .trigger	    = "$PMTK705,",	/* MKT-3301s send firmware release name and version */
+    .channels       = 12,		/* not used by this driver */
+    .control_send   = nmea_write,	/* how to send control strings */
+    .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
+    .probe_detect   = NULL,		/* no probe */
+    .probe_subtype  = NULL,		/* to be sent unconditionally */
+#ifdef ALLOW_RECONFIGURE
+    .configurator   = mkt3301_configure,	/* change its sentence set */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet     = generic_get,	/* how to get a packet */
+    .parse_packet   = nmea_parse_input,	/* how to interpret a packet */
+    .rtcm_writer    = pass_rtcm,	/* write RTCM data straight */
+    .speed_switcher = NULL,		/* no speed switcher */
+    .mode_switcher  = NULL,		/* no mode switcher */
+    .rate_switcher  = NULL,		/* no sample-rate switcher */
+    .cycle_chars    = -1,		/* not relevant, no rate switch */
+#ifdef ALLOW_RECONFIGURE
+    .revert	    = NULL,		/* no setting-reversion method */
+#endif /* ALLOW_RECONFIGURE */
+    .wrapup	    = NULL,		/* no wrapup */
+    .cycle	    = 1,		/* updates every second */
+};
+#endif /* MKT3301_ENABLE */
+
 extern struct gps_type_t garmin_usb_binary, garmin_ser_binary;
 extern struct gps_type_t sirf_binary, tsip_binary;
 extern struct gps_type_t evermore_binary, italk_binary;
-extern struct gps_type_t navcom_binary;
+extern struct gps_type_t navcom_binary, superstar2_binary;
 
 /*@ -nullassign @*/
 /* the point of this rigamarole is to not have to export a table size */
@@ -870,53 +1120,68 @@ static struct gps_type_t *gpsd_driver_array[] = {
 #ifdef ASHTECH_ENABLE
     &ashtech,
 #endif /* ASHTECHV18_ENABLE */
-#ifdef FV18_ENABLE
-    &fv18,
-#endif /* FV18_ENABLE */
-#ifdef GPSCLOCK_ENABLE
-    &gpsclock,
-#endif /* GPSCLOCK_ENABLE */
-#ifdef GARMIN_ENABLE
-    &garmin,
-#endif /* GARMIN_ENABLE */
 #ifdef TRIPMATE_ENABLE
     &tripmate,
 #endif /* TRIPMATE_ENABLE */
 #ifdef EARTHMATE_ENABLE
     &earthmate,
 #endif /* EARTHMATE_ENABLE */
+#ifdef GPSCLOCK_ENABLE
+    &gpsclock,
+#endif /* GPSCLOCK_ENABLE */
+#ifdef GARMIN_ENABLE
+    &garmin,
+#endif /* GARMIN_ENABLE */
+#ifdef MKT3301_ENABLE
+    &mkt3301,
+#endif /*  MKT3301_ENABLE */
+#ifdef OCEANSERVER_ENABLE
+    &oceanServer,
+#endif /* OCEANSERVER_ENABLE */
+#ifdef FV18_ENABLE
+    &fv18,
+#endif /* FV18_ENABLE */
+#ifdef TNT_ENABLE
+    &trueNorth,
+#endif /* TSIP_ENABLE */
 #endif /* NMEA_ENABLE */
-#ifdef ZODIAC_ENABLE
-    &zodiac_binary,
-#endif /* ZODIAC_ENABLE */
-#ifdef NAVCOM_ENABLE
-    &navcom_binary,
-#endif /* NAVCOM_ENABLE */
-#ifdef UBX_ENABLE
-    &ubx_binary,
-#endif /* UBX_ENABLE */
+
+
+#ifdef EVERMORE_ENABLE
+    &evermore_binary,
+#endif /* EVERMORE_ENABLE */
 #ifdef GARMIN_ENABLE
     &garmin_usb_binary,
     &garmin_ser_binary,
 #endif /* GARMIN_ENABLE */
+#ifdef ITRAX_ENABLE
+    &italk_binary,
+#endif /* ITRAX_ENABLE */
+#ifdef NAVCOM_ENABLE
+    &navcom_binary,
+#endif /* NAVCOM_ENABLE */
 #ifdef SIRF_ENABLE
     &sirf_binary,
 #endif /* SIRF_ENABLE */
+#ifdef SUPERSTAR2_ENABLE
+    &superstar2_binary,
+#endif /* SIRF_ENABLE */
 #ifdef TSIP_ENABLE
     &tsip_binary,
 #endif /* TSIP_ENABLE */
-#ifdef TNT_ENABLE
-    &trueNorth,
-#endif /* TSIP_ENABLE */
-#ifdef EVERMORE_ENABLE
-    &evermore_binary,
-#endif /* EVERMORE_ENABLE */
-#ifdef ITRAX_ENABLE
-    &italk_binary,
-#endif /* ITRAX_ENABLE */
-#ifdef RTCM104_ENABLE
-    &rtcm104,
-#endif /* RTCM104_ENABLE */
+#ifdef UBX_ENABLE
+    &ubx_binary,
+#endif /* UBX_ENABLE */
+#ifdef ZODIAC_ENABLE
+    &zodiac_binary,
+#endif /* ZODIAC_ENABLE */
+
+#ifdef RTCM104V2_ENABLE
+    &rtcm104v2,
+#endif /* RTCM104V2_ENABLE */
+#ifdef RTCM104V3_ENABLE
+    &rtcm104v3,
+#endif /* RTCM104V3_ENABLE */
 #ifdef GARMINTXT_ENABLE
     &garmintxt,
 #endif /* GARMINTXT_ENABLE */
