@@ -1,4 +1,4 @@
-/* $Id: libgps.c 4377 2007-06-02 14:52:38Z esr $ */
+/* $Id: libgps.c 5092 2009-01-29 00:06:04Z esr $ */
 /* libgps.c -- client interface library for the gpsd daemon */
 #include <sys/time.h>
 #include <stdio.h>
@@ -280,19 +280,22 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			gpsdata->devicelist = NULL;
 			gpsdata->ndevices = -1;
 			gpsdata->set |= DEVICELIST_SET;
-		    }    
+		    }
 		    if (sp[2] != '?') {
-			/*@ -nullderef @*/
-			gpsdata->ndevices = (int)strtol(sp+2, &sp, 10);
+			/*@ -nullderef -nullpass -mustfreeonly -dependenttrans @*/
+			char *rc = strdup(sp);
+			char *sp2 = rc;
+			gpsdata->ndevices = (int)strtol(sp2+2, &sp2, 10);
 			gpsdata->devicelist = (char **)calloc(
 			    (size_t)gpsdata->ndevices,
 			    sizeof(char **));
 			/*@ -nullstate -mustfreefresh @*/
-			gpsdata->devicelist[i=0] = strdup(strtok_r(sp+1, " \r\n", &ns));
-			while ((sp = strtok_r(NULL, " \r\n",  &ns)))
-			    gpsdata->devicelist[++i] = strdup(sp);
+			gpsdata->devicelist[i=0] = strdup(strtok_r(sp2+1, " \r\n", &ns));
+			while ((sp2 = strtok_r(NULL, " \r\n",  &ns)))
+			    gpsdata->devicelist[++i] = strdup(sp2);
+			free(rc);
 			/*@ +nullstate +mustfreefresh @*/
-			/*@ +nullderef @*/
+			/*@ +nullderef +nullpass +dependenttrans +mustfreeonly @*/
 			gpsdata->set |= DEVICELIST_SET;
 		    }
 		    break;
@@ -306,7 +309,7 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    break;
 		case 'N':
 		    if (sp[2] == '?') 
-			gpsdata->driver_mode = 0;
+			gpsdata->driver_mode = MODE_NMEA;
 		    else
 			gpsdata->driver_mode = (unsigned)atoi(sp+2);
 		    break;
@@ -557,7 +560,7 @@ int gps_query(struct gps_data_t *gpsdata, const char *fmt, ... )
 }
 
 #ifdef HAVE_LIBPTHREAD
-static void *poll_gpsd(void *args) 
+static /*@null@*/void *poll_gpsd(void *args) 
 /* helper for the thread launcher */
 {
     int oldtype, oldstate;
@@ -601,12 +604,16 @@ int gps_set_callback(struct gps_data_t *gpsdata,
 int gps_del_callback(struct gps_data_t *gpsdata, pthread_t *handler)
 /* delete asynchronous callback and kill its thread */
 {
+    /*@ -nullstate @*/
     int res;
+
     /*@i@*/res = pthread_cancel(*handler);	/* we cancel the whole thread */
+    /*@i1@*/pthread_join(*handler, NULL);	/* wait for thread to actually terminate */
     gpsdata->thread_hook = NULL;	/* finally we cancel the callback */
     if (res == 0) 			/* tell gpsd to stop sending data */
-	(void)gps_query(gpsdata,"w-\n");	/* disable watcher mode */
+	/*@i1@*/(void)gps_query(gpsdata,"w-\n");	/* disable watcher mode */
     return res;
+    /*@ +nullstate @*/
 }
 #endif /* HAVE_LIBPTHREAD */
 
@@ -669,7 +676,9 @@ static void dumpline(struct gps_data_t *ud UNUSED, char *buf,
     puts(buf);
 }
 
-#include <getopt.h>
+#ifndef S_SPLINT_S
+#include <unistd.h>
+#endif /* S_SPLINT_S */
 
 int main(int argc, char *argv[])
 {
