@@ -1,4 +1,4 @@
-/* $Id: libgpsmm.cpp 3491 2006-09-21 14:36:47Z ckuethe $ */
+/* $Id: libgpsmm.cpp 4219 2007-01-05 17:44:07Z ckuethe $ */
 /*
  * Copyright (C) 2005 Alfredo Pironti
  *
@@ -9,7 +9,7 @@
 #include "libgpsmm.h"
 
 struct gps_data_t* gpsmm::open(void) {
-	return open("localHost",DEFAULT_GPSD_PORT);
+	return open("127.0.0.1",DEFAULT_GPSD_PORT);
 }
 
 struct gps_data_t* gpsmm::open(const char *host, const char *port) {
