@@ -1,4 +1,4 @@
-/* $Id: xgps.c 4548 2007-12-13 01:40:06Z esr $ */
+/* $Id: xgps.c 5053 2009-01-21 11:44:35Z esr $ */
 /* $gpsd: xgps.c 3871 2006-11-13 00:40:00Z esr $ */
 
 /*
@@ -24,7 +24,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <math.h>
 #include <errno.h>
 #include <stdbool.h>
@@ -617,13 +619,13 @@ build_gui(Widget toplevel)
 
 /* runs when there is no data for a while */
 static void
-handle_time_out(XtPointer client_data, XtIntervalId *ignored)
+handle_time_out(XtPointer client_data UNUSED, XtIntervalId *ignored UNUSED)
 {
 	XmTextFieldSetString(text_10, "UNKNOWN");
 }
 
 static void
-handle_input(XtPointer client_data, int *source, XtInputId *id)
+handle_input(XtPointer client_data UNUSED, int *source UNUSED, XtInputId *id UNUSED)
 {
 	if (gps_poll(gpsdata) < 0) {
 		XtRemoveInput(gps_input);
@@ -802,7 +804,7 @@ get_resource(Widget w, char *name, char *default_value)
 /* runs when gps needs attention */
 /*@ -globstate -branchstate @*/
 void
-handle_gps(XtPointer client_data, XtIntervalId *ignored)
+handle_gps(XtPointer client_data UNUSED, XtIntervalId *ignored UNUSED)
 {
 	char *err_str = NULL;
 	char error[128];
@@ -899,13 +901,13 @@ err_dialog(Widget widget, char *s)
 }
 
 void
-dlg_callback(Widget dialog, XtPointer client_data, XtPointer call_data)
+dlg_callback(Widget dialog, XtPointer client_data UNUSED, XtPointer call_data UNUSED)
 {
     /*@i1@*/XtPopdown(XtParent(dialog));
 }
 
 void
-file_cb(Widget widget, XtPointer client_data, XtPointer call_data)
+file_cb(Widget widget UNUSED, XtPointer client_data, XtPointer call_data UNUSED)
 {
 	uintptr_t item_no = (uintptr_t)client_data;
 
@@ -914,7 +916,7 @@ file_cb(Widget widget, XtPointer client_data, XtPointer call_data)
 }
 
 void
-help_cb(Widget widget, XtPointer client_data, XtPointer call_data)
+help_cb(Widget widget UNUSED, XtPointer client_data, XtPointer call_data UNUSED)
 {
 	static Widget help, about;
 	Widget *dialog;
@@ -958,7 +960,7 @@ help_cb(Widget widget, XtPointer client_data, XtPointer call_data)
 		XtUnmanageChild(XmMessageBoxGetChild(about,
 		    XmDIALOG_HELP_BUTTON));
 	}
-	/*@ +usedef +immediatetrans +onlytrans +mustfreefresh +type -charint +ptrcompare @*/
+	/*@ +usedef +immediatetrans +onlytrans +type -charint +ptrcompare @*/
 
 	if (item_no == 0)
 		dialog = &help;
@@ -966,7 +968,8 @@ help_cb(Widget widget, XtPointer client_data, XtPointer call_data)
 		dialog = &about;
 
 	XtManageChild(*dialog);
-	/*@i1@*/XtPopup(XtParent(*dialog), XtGrabNone);
+	XtPopup(XtParent(*dialog), XtGrabNone);
+	/*@ +mustfreefresh @*/
 }
 
 /*@ -mustfreefresh @*/
