@@ -1,18 +1,19 @@
-/* $Id: xgpsspeed.c 3666 2006-10-26 23:11:51Z ckuethe $ */
+/* $Id: xgpsspeed.c 5053 2009-01-21 11:44:35Z esr $ */
 /* GPS speedometer as a wrapper around an Athena widget Tachometer
  * - Derrick J Brashear <shadow@dementia.org>
  */
 #include <sys/types.h>
 #include <stdlib.h>
+#ifndef S_SPLINT_S
 #include <unistd.h>
+#endif /* S_SPLINT_S */
 #include <stdio.h>
 #include <math.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
+#include <X11/StringDefs.h>
 #include <X11/Xaw/Label.h>
 #include <X11/Xaw/Paned.h>
-#include <Xm/Xm.h>
-#include <Xm/XmStrDefs.h>
 #include <Tachometer.h>
 
 #include "gpsd_config.h"
@@ -63,10 +64,10 @@ static char *get_resource(Widget w, char *name, char *default_value)
   /*@ -observertrans -statictrans -immediatetrans -compdestroy -nullpass @*/
   xtr.resource_name = name;
   xtr.resource_class = "AnyClass";
-  xtr.resource_type = XmRString;
+  xtr.resource_type = XtRString;
   xtr.resource_size = (Cardinal)sizeof(String);
   xtr.resource_offset = 0;
-  xtr.default_type = XmRImmediate;
+  xtr.default_type = XtRImmediate;
   xtr.default_addr = default_value;
   XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
   if (value) return value;
@@ -154,7 +155,7 @@ int main(int argc, char **argv)
     else 
         (void)XtSetArg(args[0], XtNlabel, "Knots");
 
-    /*@ +immediatetrans +usedef +observertrans +statictrans @*/
+    /*@ +immediatetrans +usedef +observertrans +statictrans -compmempass @*/
     (void)XtCreateManagedWidget("name", labelWidgetClass, base, args, 1);
     
     /**** Tachometer widget ****/
